#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>


void show_usage(void);
char *randompasswd(int len, char *string);

struct config_struct {
	unsigned digits	: 2;
	unsigned lower	: 2;
	unsigned upper	: 2;
	unsigned special : 2;
	unsigned int number;
	unsigned int len;
};

struct config_struct config;

int main(int argc, char *argv[])
{

	/* Initializing variables */
	int option, a;
	char *passwords;
	char digits[] = "0123456789";
	char lowletters[] = "abcdefghijklmnopqrstuvwxyz";
	char upletters[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	char spechar[] = "![]&%?$@(){}-_.";
	char *stringchar = malloc(1);;

	config.digits = 0;
        config.lower = 0;
	config.upper = 0;
	config.special = 0;
        config.len = 8;

	/* check for options */
	while ((option= getopt (argc,argv,"hdaApsn:l:")) != EOF)
	{
		switch (option)
		{
			case 'd' :
				if (config.digits < 3)
					config.digits++;
				break;
			case 'a' :
				if (config.lower < 3)
					config.lower++;
				break;
			case 'A' :
				if (config.upper < 3)
					config.upper++;
				break;
			case 'p' :
				config.digits = 1;
				config.lower = 1;
				config.upper = 1;
				config.special = 1;
				config.len = 12;
				break;
			case 's' :
				if (config.special < 3)
					config.special++;
				break;
			case 'l' :
				config.len = atoi(optarg);
				break;
			case 'n' :
				config.number = atoi(optarg);
				break;
			case 'h' :
				show_usage();
				exit(0);
				break;
			default :
				printf("Unknown option!\n\n-----\n");
				show_usage();
				exit(1);
		}
	}

	
	/* check for errors with len (-l) and numbers (-n) */
	if (config.len < 1)
		config.len = 8;

	if (config.digits == 0 && config.lower == 0 && config.upper == 0 && config.special == 0)
        {
                        config.digits = 1;
                        config.lower = 1;
        }


	if (config.number < 1 || config.number > 1000)
		config.number = 3;
	puts("Generating passwords . . .");

	
	if (config.digits)
	{
		int len = strlen(digits);
		
		for (a=0; a<config.digits; a++)
		{
			stringchar = realloc(stringchar,(strlen(stringchar) + sizeof(char)*len));
			strncat(stringchar,digits,len);
		}
		
	}
	
	if (config.lower)
	{
		int len = strlen(lowletters);
                for (a=0; a<config.lower; a++)
                {
			stringchar = realloc(stringchar,(strlen(stringchar) + sizeof(char)*len));
			strncat(stringchar,lowletters,len);
		}
	}
	
	if (config.upper)
	{
		int len = strlen(upletters);
                for (a=0; a<config.upper; a++)
                {
			stringchar = realloc(stringchar,(strlen(stringchar) + sizeof(char)*len));
			strncat(stringchar,upletters,len);
		}
	}
	
	if (config.special)
	{
		int len = strlen(spechar);
                for (a=0; a<config.special; a++)
                {
			stringchar = realloc(stringchar,(strlen(stringchar) + sizeof(char)*len));
			strncat(stringchar,spechar,len);
		}
	}

	/* Generating passwords */	
	for (a = 0; a < config.number; a++) 
	{
		passwords = randompasswd(config.len,stringchar);	
		printf("\nPassword %d\t: %s",a + 1, passwords);
		/* Use sleep due to srand initialization that is using the time */
		sleep(1);
	}	
	puts("\n");
	printf("%s (c) Jean-François Lévesque, 2001\n",argv[0]);
	return 0;
}


void show_usage(void)
{
	puts("\nUsage: passwd-gen <OPTIONS>...");
	puts("  -a \t\t: include lower case letters in the password");
	puts("  -A \t\t: include upper case letters in the password");
	puts("  -d \t\t: include digits in the password");
	puts("  -s \t\t: include special characters in the password");
	puts("  -p \t\t: generate a powerful password (same as -aAdsl 12)");
	puts("  -l <lenght>\t: lenght of the password (default : 8)");
	puts("  -n <number>\t: number of password to generate (default : 3)");
	puts("\nIf you want more digits, lowercase letters, uppercase letters");
	puts("or special characters, use the option 2 or 3 times.\n");
	puts("Example:\npasswd-gen -addd\n");
}


/*
 * Generate a random combination!
 */

char *randompasswd(int len, char *string)
{

	int b,randnb;
	char randomchar[strlen(string)];
	char randompasswd[len+1];
	char *randompasswdptr;
	float len2= strlen(string) - 1;
	randompasswdptr = malloc(sizeof(char)*(len+1));

	/* initialize random sequence */	
	srand(time(NULL)+getpid());
	
	/* set into randomchar all the valid characters */
	sprintf(randomchar,"%s",string);

	/* setting random value to each character of the activation key */
	for (b=0;b <= len;b++) {
		randnb=(int) (len2*rand()/(RAND_MAX+1.0));
		randnb=(int) (len2*rand()/(RAND_MAX+1.0));
		randompasswd[b] = randomchar[randnb];
	}

	/* setting the null character */
	randompasswd[len] = 0;

	sprintf(randompasswdptr,"%s",randompasswd);
	/* Return the char pointer */
	return randompasswdptr;
}

